#    Copyright 2019, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_config import cfg
from oslo_log import log as logging
from taskflow import task

from a10_octavia.common import a10constants
from a10_octavia.common import openstack_mappings
from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
from a10_octavia.controller.worker.tasks import utils

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class CreateAndAssociateHealthMonitor(task.Task):
    """Task to create a healthmonitor and associate it with provided pool."""

    @axapi_client_decorator
    def execute(self, health_mon, vthunder):
        method = None
        url = None
        expect_code = None
        if health_mon.type in a10constants.HTTP_TYPE:
            method = health_mon.http_method
            url = health_mon.url_path
            expect_code = health_mon.expected_codes
        args = utils.meta(health_mon, 'hm', {})

        try:
            self.axapi_client.slb.hm.create(health_mon.id[0:5],
                                            openstack_mappings.hm_type(self.axapi_client,
                                            health_mon.type),
                                            health_mon.delay, health_mon.timeout,
                                            health_mon.rise_threshold, method=method, url=url,
                                            expect_code=expect_code, axapi_args=args)
            LOG.debug("Health Monitor created successfully: %s", health_mon.id)
        except Exception as e:
            LOG.exception("Failed to create Health Monitor: %s", str(e))
            raise

        try:
            self.axapi_client.slb.service_group.update(health_mon.pool_id,
                                                       health_monitor=health_mon.id[0:5],
                                                       health_check_disable=0)
            LOG.debug("Health Monitor %s is associated to pool %s successfully.",
                      health_mon.id, health_mon.pool_id)
        except Exception as e:
            LOG.exception("Failed to associate pool to Health Monitor: %s", str(e))
            raise


class DeleteHealthMonitor(task.Task):
    """Task to disassociate Health Monitor from pool and delete"""

    @axapi_client_decorator
    def execute(self, health_mon, vthunder):
        try:
            self.axapi_client.slb.service_group.update(health_mon.pool_id,
                                                       health_monitor="",
                                                       health_check_disable=True)
            LOG.debug("Health Monitor %s is dissociated from pool %s successfully.",
                      health_mon.id, health_mon.pool_id)
        except Exception as e:
            LOG.warning("Failed to dissociate Health Monitor from pool: %s", str(e))
        try:
            self.axapi_client.slb.hm.delete(health_mon.id[0:5])
            LOG.debug("Health Monitor deleted successfully: %s", health_mon.id)
        except Exception as e:
            LOG.warning("Failed to delete health monitor: %s", str(e))


class UpdateHealthMonitor(task.Task):
    """Task to update Health Monitor"""

    @axapi_client_decorator
    def execute(self, health_mon, vthunder, update_dict):
        """ Execute update health monitor """
        # TODO(hthompson6) Length of name of healthmonitor for older vThunder devices
        health_mon.__dict__.update(update_dict)
        method = None
        url = None
        expect_code = None
        if health_mon.type in a10constants.HTTP_TYPE:
            method = health_mon.http_method
            url = health_mon.url_path
            expect_code = health_mon.expected_codes
        args = utils.meta(health_mon, 'hm', {})
        try:
            self.axapi_client.slb.hm.update(
                health_mon.id[0:5],
                openstack_mappings.hm_type(self.axapi_client, health_mon.type),
                health_mon.delay, health_mon.timeout, health_mon.rise_threshold,
                method=method, url=url, expect_code=expect_code, axapi_args=args)
            LOG.debug("Health Monitor updated successfully: %s", health_mon.id)
        except Exception as e:
            LOG.exception("Failed to update Health Monitor: %s", str(e))



#    Copyright 2019, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log as logging
from taskflow import task

import acos_client.errors as acos_errors
from octavia.certificates.common.auth.barbican_acl import BarbicanACLAuth

from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
from a10_octavia.controller.worker.tasks import utils

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class ListenersParent(object):

    def set(self, set_method, loadbalancer, listeners):
        ipinip = CONF.listener.ipinip
        no_dest_nat = CONF.listener.no_dest_nat
        autosnat = CONF.listener.autosnat
        conn_limit = CONF.listener.conn_limit
        virtual_port_templates = {}
        template_virtual_port = CONF.listener.template_virtual_port
        virtual_port_templates['template-virtual-port'] = template_virtual_port

        template_args = {}
        try:
            for listener in listeners:
                if listener.connection_limit != -1:
                    conn_limit = listener.connection_limit
                if conn_limit < 1 or conn_limit > 64000000:
                    LOG.warning("The specified member server connection limit " +
                                "(configuration setting: conn-limit) is out of " +
                                "bounds with value {0}. Please set to between " +
                                "1-64000000. Defaulting to 64000000".format(conn_limit))
                listener.load_balancer = loadbalancer
                status = self.axapi_client.slb.UP
                if not listener.enabled:
                    status = self.axapi_client.slb.DOWN
                c_pers, s_pers = utils.get_sess_pers_templates(listener.default_pool)
                if listener.protocol == "TERMINATED_HTTPS":
                    listener.protocol = 'HTTPS'
                    template_args["template_client_ssl"] = self.cert_handler(
                        loadbalancer, listener)

                if listener.protocol.lower() == 'http':
                    # TODO(hthompson6) work around for issue in acos client
                    listener.protocol = listener.protocol.lower()
                    virtual_port_template = CONF.listener.template_http
                    virtual_port_templates['template-http'] = virtual_port_template
                else:
                    virtual_port_template = CONF.listener.template_tcp
                    virtual_port_templates['template-tcp'] = virtual_port_template

                virtual_port_template = CONF.listener.template_policy
                virtual_port_templates['template-policy'] = virtual_port_template

                name = loadbalancer.id + "_" + str(listener.protocol_port)
                set_method(loadbalancer.id, name,
                           listener.protocol,
                           listener.protocol_port,
                           listener.default_pool_id,
                           s_pers_name=s_pers, c_pers_name=c_pers,
                           status=status, no_dest_nat=no_dest_nat,
                           autosnat=autosnat, ipinip=ipinip,
                           # TODO(hthompson6) resolve in acos client
                           # ha_conn_mirror=ha_conn_mirror,
                           conn_limit=conn_limit,
                           virtual_port_templates=virtual_port_templates,
                           **template_args)
                LOG.info("Listener created successfully.")
        except Exception as e:
            LOG.error(str(e))
            LOG.info("Error occurred")

    def cert_handler(self, loadbalancer, listener):
        """Function to handle TLS certs"""
        cert_data = dict()
        bauth = BarbicanACLAuth()
        barbican_client = bauth.get_barbican_client(loadbalancer.project_id)
        container = barbican_client.containers.get(container_ref=listener.tls_certificate_id)

        cert_data["cert_content"] = container.certificate.payload
        cert_data["key_content"] = container.private_key.payload
        cert_data["key_pass"] = container.private_key_passphrase
        cert_data["template_name"] = listener.id
        cert_data["cert_filename"] = container.certificate.name
        cert_data["key_filename"] = container.private_key.name

        try:
            self.axapi_client.file.ssl_cert.create(file=cert_data["cert_filename"],
                                                   cert=cert_data["cert_content"],
                                                   size=len(cert_data["cert_content"]),
                                                   action="import", certificate_type="pem")
        except acos_errors.Exists:
            self.axapi_client.file.ssl_cert.update(file=cert_data["cert_filename"],
                                                   cert=cert_data["cert_content"],
                                                   size=len(cert_data["cert_content"]),
                                                   action="import", certificate_type="pem")
        try:
            self.axapi_client.file.ssl_key.create(file=cert_data["key_filename"],
                                                  cert=cert_data["key_content"],
                                                  size=len(cert_data["key_content"]),
                                                  action="import")
        except acos_errors.Exists:
            self.axapi_client.file.ssl_key.update(file=cert_data["key_filename"],
                                                  cert=cert_data["key_content"],
                                                  size=len(cert_data["key_content"]),
                                                  action="import")
        # create template
        try:
            self.axapi_client.slb.template.client_ssl.create(cert_data["template_name"],
                                                             cert=cert_data["cert_filename"],
                                                             key=cert_data["key_filename"],
                                                             passphrase=cert_data["key_pass"])
        except acos_errors.Exists:
            self.axapi_client.slb.template.client_ssl.update(cert_data["template_name"],
                                                             cert=cert_data["cert_filename"],
                                                             key=cert_data["key_filename"],
                                                             passphrase=cert_data["key_pass"])

        return cert_data["template_name"]


class ListenersCreate(ListenersParent, task.Task):

    """Task to create listener"""

    @axapi_client_decorator
    def execute(self, loadbalancer, listeners, vthunder):
        self.set(self.axapi_client.slb.virtual_server.vport.create, loadbalancer, listeners)

    def revert(self, loadbalancer, *args, **kwargs):
        """ Handle failed listeners updates """
        LOG.warning("Reverting listeners updates.")

        for listener in loadbalancer.listeners:
            self.task_utils.mark_listener_prov_status_error(listener.id)

        return None


class ListenersUpdate(ListenersParent, task.Task):

    """Task to update listener"""

    @axapi_client_decorator
    def execute(self, loadbalancer, listeners, vthunder):
        self.set(self.axapi_client.slb.virtual_server.vport.update, loadbalancer, listeners)

    def revert(self, loadbalancer, *args, **kwargs):
        """ Handle failed listeners updates """
        LOG.warning("Reverting listeners updates.")

        for listener in loadbalancer.listeners:
            self.task_utils.mark_listener_prov_status_error(listener.id)

        return None


class ListenerDelete(task.Task):

    """Task to delete the listener"""

    @axapi_client_decorator
    def execute(self, loadbalancer, listener, vthunder):
        name = loadbalancer.id + "_" + str(listener.protocol_port)
        try:
            self.axapi_client.slb.virtual_server.vport.delete(
                loadbalancer.id, name, listener.protocol,
                listener.protocol_port)
            LOG.debug("Listener deleted successfully: %s", name)
        except Exception as e:
            LOG.warning("Failed to delete the listener: %s", str(e))

    def revert(self, listener, *args, **kwargs):
        """ Handle a failed listener delete """
        LOG.warning("Reverting listener delete.")

        self.task_utils.mark_listener_prov_status_error(listener.id)



#    Copyright 2019, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log as logging
from taskflow import task

from a10_octavia.common import openstack_mappings
from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
from a10_octavia.controller.worker.tasks import utils

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class PoolParent(object):

    def set(self, set_method, pool):

        axapi_args = {'service_group': utils.meta(pool, 'service_group', {})}
        service_group_temp = {}
        service_group_temp['template-server'] = CONF.service_group.template_server
        service_group_temp['template-port'] = CONF.service_group.template_port
        service_group_temp['template-policy'] = CONF.service_group.template_policy
        protocol = openstack_mappings.service_group_protocol(self.axapi_client, pool.protocol)
        lb_method = openstack_mappings.service_group_lb_method(self.axapi_client, pool.lb_algorithm)
        set_method(pool.id,
                   protocol=protocol,
                   lb_method=lb_method,
                   service_group_templates=service_group_temp,
                   axapi_args=axapi_args)


class PoolCreate(PoolParent, task.Task):
    """Task to create pool"""

    @axapi_client_decorator
    def execute(self, pool, vthunder):
        try:
            self.set(self.axapi_client.slb.service_group.create, pool)
            LOG.debug("Pool created successfully: %s", pool.id)
            return pool
        except Exception as e:
            LOG.exception("Failed to create pool: %s", str(e))
            raise

    @axapi_client_decorator
    def revert(self, pool, vthunder, *args, **kwargs):
        try:
            self.axapi_client.slb.service_group.delete(pool.id)
        except Exception as e:
            LOG.warning("Failed to revert create pool: %s", str(e))


class PoolDelete(task.Task):
    """Task to delete pool"""

    @axapi_client_decorator
    def execute(self, pool, vthunder):
        try:
            self.axapi_client.slb.service_group.delete(pool.id)
            LOG.debug("Pool deleted successfully: %s", pool.id)
        except Exception as e:
            LOG.warning("Failed to delete pool: %s", str(e))


class PoolUpdate(PoolParent, task.Task):
    """Task to update pool"""

    @axapi_client_decorator
    def execute(self, pool, vthunder, update_dict):
        pool.__dict__.update(update_dict)
        try:
            self.set(self.axapi_client.slb.service_group.update, pool)
            LOG.debug("Pool updated successfully: %s", pool.id)
        except Exception as e:
            LOG.warning("Failed to update pool: %s", str(e))



#    Copyright 2019, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from datetime import datetime
import sqlalchemy
from sqlalchemy.orm.exc import NoResultFound

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils
from taskflow import task

from octavia.common import constants
from octavia.db import api as db_apis
from octavia.db import repositories as repo

from a10_octavia.db import repositories as a10_repo

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class BaseDatabaseTask(task.Task):

    """Base task to load drivers common to the tasks."""

    def __init__(self, **kwargs):
        self.repos = repo.Repositories()
        self.vthunder_repo = a10_repo.VThunderRepository()
        self.amphora_repo = repo.AmphoraRepository()
        super(BaseDatabaseTask, self).__init__(**kwargs)


class GetVThunderTask(BaseDatabaseTask):

    """Test VThunder entry"""

    def execute(self, amphora):
        vthunder = self.vthunder_repo.get(db_apis.get_session(), id=123)
        return vthunder


class CreateVThunderEntry(BaseDatabaseTask):

    """ Create VThunder device entry in DB"""

    def execute(self, amphora, loadbalancer, role, status):
        vthunder_id = uuidutils.generate_uuid()

        username = CONF.vthunder.default_vthunder_username
        password = CONF.vthunder.default_vthunder_password
        axapi_version = CONF.vthunder.default_axapi_version

        if role == constants.ROLE_MASTER:
            topology = "ACTIVE_STANDBY"
            role = "MASTER"
        elif role == constants.ROLE_BACKUP:
            topology = "ACTIVE_STANDBY"
            role = "BACKUP"
        else:
            topology = "STANDALONE"
            role = "MASTER"

        self.vthunder_repo.create(
            db_apis.get_session(), vthunder_id=vthunder_id,
            amphora_id=amphora.id,
            device_name=vthunder_id, username=username,
            password=password, ip_address=amphora.lb_network_ip,
            undercloud=False, axapi_version=axapi_version,
            loadbalancer_id=loadbalancer.id,
            project_id=loadbalancer.project_id,
            compute_id=amphora.compute_id,
            topology=topology,
            role=role,
            last_udp_update=datetime.utcnow(),
            status=status,
            created_at=datetime.utcnow(),
            updated_at=datetime.utcnow())

        LOG.info("Successfully created vthunder entry in database.")

    def revert(self, amphora, loadbalancer, role, status=constants.ACTIVE,
               *args, **kwargs):
        try:
            self.vthunder_repo.delete(
                db_apis.get_session(), loadbalancer_id=loadbalancer.id)
        except NoResultFound:
            LOG.error("Failed to delete vThunder entry for load balancer: %s", loadbalancer.id)


class DeleteVThunderEntry(BaseDatabaseTask):

    """ Delete VThunder device entry in DB  """

    def execute(self, loadbalancer):
        try:
            self.vthunder_repo.delete(
                db_apis.get_session(), loadbalancer_id=loadbalancer.id)
        except NoResultFound:
            pass
        LOG.info("Successfully deleted vthunder entry in database.")


class GetVThunderByLoadBalancer(BaseDatabaseTask):

    """ Get VThunder details from LoadBalancer"""

    def execute(self, loadbalancer):
        loadbalancer_id = loadbalancer.id
        vthunder = self.vthunder_repo.get_vthunder_from_lb(
            db_apis.get_session(), loadbalancer_id)
        LOG.info("Successfully fetched vThunder details for LB")
        return vthunder


class GetBackupVThunderByLoadBalancer(BaseDatabaseTask):

    """ Get VThunder details from LoadBalancer"""

    def execute(self, loadbalancer):
        loadbalancer_id = loadbalancer.id
        vthunder = self.vthunder_repo.get_backup_vthunder_from_lb(
            db_apis.get_session(), loadbalancer_id)
        return vthunder
        LOG.info("Successfully fetched vThunder details for LB")

# class GetVThunderByLoadBalancerID(BaseDatabaseTask):
#     """ Get VThunder details from LoadBalancer ID """
#     def execute(self, loadbalancer_id):
#         vthunder = self.vthunder_repo.get_vthunder_from_lb(db_apis.get_session(), loadbalancer_id)
#         return vthunder
#         LOG.info("Successfully fetched vThunder details for LB")


class GetComputeForProject(BaseDatabaseTask):

    """ Get Compute details form Loadbalancer object -> project ID"""

    def execute(self, loadbalancer):
        vthunder = self.vthunder_repo.get_vthunder_by_project_id(
            db_apis.get_session(), loadbalancer.project_id)
        if vthunder is None:
            vthunder = self.vthunder_repo.get_spare_vthunder(
                db_apis.get_session())
            self.vthunder_repo.update(db_apis.get_session(), vthunder.id,
                                      status="USED_SPARE", updated_at=datetime.utcnow())
        amphora_id = vthunder.amphora_id
        self.amphora_repo.get(db_apis.get_session(), id=amphora_id)
        compute_id = vthunder.compute_id
        return compute_id


class MapLoadbalancerToAmphora(BaseDatabaseTask):

    """Maps and assigns a load balancer to an amphora in the database."""

    def execute(self, loadbalancer, server_group_id=None):
        """Allocates an Amphora for the load balancer in the database.

        :param loadbalancer_id: The load balancer id to map to an amphora
        :returns: Amphora ID if one was allocated, None if it was
                  unable to allocate an Amphora
        """

        if server_group_id is not None:
            LOG.debug("Load balancer is using anti-affinity. Skipping spares "
                      "pool allocation.")
            return None

        vthunder = self.vthunder_repo.get_vthunder_by_project_id(
            db_apis.get_session(),
            loadbalancer.project_id)

        if vthunder is None:
            # Check for spare vthunder
            vthunder = self.vthunder_repo.get_spare_vthunder(db_apis.get_session())
            if vthunder is None:
                LOG.debug("No Amphora available for load balancer with id %s",
                          loadbalancer.id)
                return None

        return vthunder.id


class CreateRackVthunderEntry(BaseDatabaseTask):

    """ Create VThunder device entry in DB """

    def execute(self, loadbalancer, vthunder_config):
        self.vthunder_repo.create(db_apis.get_session(),
                                  vthunder_id=uuidutils.generate_uuid(),
                                  device_name=vthunder_config.device_name,
                                  username=vthunder_config.username,
                                  password=vthunder_config.password,
                                  ip_address=vthunder_config.ip_address,
                                  undercloud=vthunder_config.undercloud,
                                  loadbalancer_id=loadbalancer.id,
                                  project_id=vthunder_config.project_id,
                                  axapi_version=vthunder_config.axapi_version,
                                  topology="STANDALONE",
                                  role="MASTER",
                                  status="ACTIVE",
                                  last_udp_update=datetime.utcnow(),
                                  created_at=datetime.utcnow(),
                                  updated_at=datetime.utcnow())
        LOG.info("Successfully created vthunder entry in database.")


class CreateVThunderHealthEntry(BaseDatabaseTask):

    """ Create VThunder Health entry in DB """

    def execute(self, loadbalancer, vthunder_config):
        self.vthunder_repo.create(db_apis.get_session(),
                                  vthunder_id=uuidutils.generate_uuid(),
                                  device_name=vthunder_config.device_name,
                                  username=vthunder_config.username,
                                  password=vthunder_config.password,
                                  ip_address=vthunder_config.ip_address,
                                  undercloud=vthunder_config.undercloud,
                                  loadbalancer_id=loadbalancer.id,
                                  project_id=vthunder_config.project_id,
                                  axapi_version=vthunder_config.axapi_version,
                                  topology="STANDALONE",
                                  role="MASTER")
        LOG.info("Successfully created vthunder entry in database.")


class MarkVThunderStatusInDB(BaseDatabaseTask):

    def execute(self, vthunder, status):
        try:
            if vthunder:
                self.vthunder_repo.update(db_apis.get_session(),
                                          vthunder.id,
                                          status=status, updated_at=datetime.utcnow())
        except (sqlalchemy.orm.exc.NoResultFound,
                sqlalchemy.orm.exc.UnmappedInstanceError):
            LOG.debug('No existing amphora health record to mark busy '
                      'for amphora: %s, skipping.', vthunder.id)


class CreateSpareVThunderEntry(BaseDatabaseTask):

    def execute(self, amphora):
        vthunder_id = uuidutils.generate_uuid()

        username = CONF.vthunder.default_vthunder_username
        password = CONF.vthunder.default_vthunder_password
        axapi_version = CONF.vthunder.default_axapi_version
        vthunder = self.vthunder_repo.create(
            db_apis.get_session(), vthunder_id=vthunder_id,
            amphora_id=amphora.id,
            device_name=vthunder_id, username=username,
            password=password, ip_address=amphora.lb_network_ip,
            undercloud=False, axapi_version=axapi_version,
            loadbalancer_id=None,
            project_id=None,
            compute_id=amphora.compute_id,
            topology="SINGLE",
            role="MASTER",
            status="READY",
            created_at=datetime.utcnow(),
            last_udp_update=datetime.utcnow(),
            updated_at=datetime.utcnow())
        LOG.info("Successfully created vthunder entry in database.")
        return vthunder



#    Copyright 2019, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from a10_octavia.db import models
from datetime import datetime

models.VThunder.create_and_save(amphora_id_id='112', device_name='somename',
                                ip_address='192.168.2.3', username='username',
                                password='password', axapi_version=30,
                                undercloud=True, loadbalancer_id='lbid',
                                status='ACTIVE', updated_at=datetime.utcnow(),
                                created_at=datetime.utcnow(),
                                last_udp_update=datetime.utcnow(),
                                db_session=None)



"""updates

Revision ID: 2d0bd8e6648f
Revises: 3446633d0749
Create Date: 2019-06-28 15:26:52.227260

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '2d0bd8e6648f'
down_revision = '3446633d0749'
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        'vthunders',
        sa.Column('id', sa.Integer, primary_key=True),
        sa.Column('vthunder_id', sa.String(36), nullable=False),
        sa.Column('amphora_id', sa.String(36), nullable=True),
        sa.Column('device_name', sa.String(1024), nullable=False),
        sa.Column('ip_address', sa.String(64), nullable=False),
        sa.Column('username', sa.String(1024), nullable=False),
        sa.Column('password', sa.String(50), nullable=False),
        sa.Column('axapi_version', sa.Integer, default=30, nullable=False),
        sa.Column('undercloud', sa.Boolean(), default=False, nullable=False),
        sa.Column('loadbalancer_id', sa.String(36)),
        sa.Column('project_id', sa.String(36))
    )


def downgrade():
    pass



#    Copyright 2020, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import logging
from taskflow import task

from octavia.certificates.common.auth.barbican_acl import BarbicanACLAuth

from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
from a10_octavia.controller.worker.tasks import utils

LOG = logging.getLogger(__name__)


class SSLCertCreate(task.Task):
    """Task to create an SSL certificate"""
    @axapi_client_decorator
    def execute(self, loadbalancer, listener, vthunder, certificate_type="pem"):
        barbican_client = BarbicanACLAuth().get_barbican_client(loadbalancer.project_id)
        cert_data = utils.get_cert_data(barbican_client, listener)
        try:
            if self.axapi_client.file.ssl_cert.exists(file=cert_data.cert_filename):
                self.axapi_client.file.ssl_cert.update(file=cert_data.cert_filename,
                                                       cert=cert_data.cert_content,
                                                       size=len(cert_data.cert_content),
                                                       action="import",
                                                       certificate_type=certificate_type)
            else:
                self.axapi_client.file.ssl_cert.create(file=cert_data.cert_filename,
                                                       cert=cert_data.cert_content,
                                                       size=len(cert_data.cert_content),
                                                       action="import",
                                                       certificate_type=certificate_type)
        except Exception as e:
            LOG.exception("Failed to create SSL certificate: %s", str(e))
            raise e
        return cert_data

    @axapi_client_decorator
    def revert(self, loadbalancer, listener, vthunder, *args, **kwargs):
        barbican_client = BarbicanACLAuth().get_barbican_client(loadbalancer.project_id)
        cert_data = utils.get_cert_data(barbican_client, listener)
        try:
            self.axapi_client.file.ssl_cert.delete(file=cert_data.cert_filename)
            LOG.debug("Reverted SSLCertCreate: %s", cert_data.cert_filename)
        except Exception:
            pass


class SSLKeyCreate(task.Task):
    """Task to create an SSL Key"""
    @axapi_client_decorator
    def execute(self, cert_data, vthunder):
        try:
            if self.axapi_client.file.ssl_key.exists(file=cert_data.key_filename):
                self.axapi_client.file.ssl_key.update(file=cert_data.key_filename,
                                                      cert=cert_data.key_content,
                                                      size=len(cert_data.key_content),
                                                      action="import")
            else:
                self.axapi_client.file.ssl_key.create(file=cert_data.key_filename,
                                                      cert=cert_data.key_content,
                                                      size=len(cert_data.key_content),
                                                      action="import")
        except Exception as e:
            LOG.exception("Failed to create SSL key: %s", str(e))
            raise e

    @axapi_client_decorator
    def revert(self, cert_data, vthunder, *args, **kwargs):
        try:
            self.axapi_client.file.ssl_key.delete(file=cert_data.key_filename)
            LOG.debug("Reverted SSLKeyCreate: %s", cert_data.key_filename)
        except Exception:
            pass


class ClientSSLTemplateCreate(task.Task):
    """Task to create a client ssl template for a listener"""
    @axapi_client_decorator
    def execute(self, cert_data, vthunder):
        try:
            if self.axapi_client.slb.template.client_ssl.exists(name=cert_data.template_name):
                self.axapi_client.slb.template.client_ssl.update(name=cert_data.template_name,
                                                                 cert=cert_data.cert_filename,
                                                                 key=cert_data.key_filename,
                                                                 passphrase=cert_data.key_pass)
            else:
                self.axapi_client.slb.template.client_ssl.create(name=cert_data.template_name,
                                                                 cert=cert_data.cert_filename,
                                                                 key=cert_data.key_filename,
                                                                 passphrase=cert_data.key_pass)
        except Exception as e:
            LOG.exception("Failed to create client SSL template: %s", str(e))
            raise e

    @axapi_client_decorator
    def revert(self, cert_data, vthunder, *args, **kwargs):
        try:
            self.axapi_client.slb.template.client_ssl.delete(name=cert_data.template_name)
            LOG.debug("Reverted ClientSSLTemplateCreate: %s", cert_data.template_name)
        except Exception:
            pass


class SSLCertUpdate(task.Task):
    """Task to update an SSL certificate"""
    @axapi_client_decorator
    def execute(self, loadbalancer, listener, vthunder, action="import", certificate_type="pem"):
        barbican_client = BarbicanACLAuth().get_barbican_client(loadbalancer.project_id)
        cert_data = utils.get_cert_data(barbican_client, listener)
        try:
            if self.axapi_client.file.ssl_cert.exists(file=cert_data.cert_filename):
                self.axapi_client.file.ssl_cert.update(file=cert_data.cert_filename,
                                                       cert=cert_data.cert_content,
                                                       size=len(cert_data.cert_content),
                                                       action=action,
                                                       certificate_type=certificate_type)
                LOG.debug("SSL certificate updated successfully: %s", cert_data.cert_filename)
        except Exception as e:
            LOG.warning("Failed to update SSL Certificate: %s", str(e))
        return cert_data


class SSLKeyUpdate(task.Task):
    """Task to update an SSL Key"""
    @axapi_client_decorator
    def execute(self, cert_data, vthunder, action="import"):
        try:
            if self.axapi_client.file.ssl_key.exists(file=cert_data.key_filename):
                self.axapi_client.file.ssl_key.update(file=cert_data.key_filename,
                                                      cert=cert_data.key_content,
                                                      size=len(cert_data.key_content),
                                                      action=action)
                LOG.debug("SSL Key updated successfully: %s", cert_data.key_filename)
        except Exception as e:
            LOG.warning("Failed to update SSL Key: %s", str(e))


class ClientSSLTemplateUpdate(task.Task):
    """Task to update a client ssl template for a listener"""
    @axapi_client_decorator
    def execute(self, cert_data, vthunder):
        try:
            if self.axapi_client.slb.template.client_ssl.exists(name=cert_data.template_name):
                self.axapi_client.slb.template.client_ssl.update(name=cert_data.template_name,
                                                                 cert=cert_data.cert_filename,
                                                                 key=cert_data.key_filename,
                                                                 passphrase=cert_data.key_pass)
                LOG.debug("Client SSL Template updated successfully: %s",
                          cert_data.template_name)
        except Exception as e:
            LOG.warning("Failed to update Client SSL Template: %s", str(e))


class SSLCertDelete(task.Task):
    """Task to delete an SSL certificate"""
    @axapi_client_decorator
    def execute(self, loadbalancer, listener, vthunder):
        barbican_client = BarbicanACLAuth().get_barbican_client(loadbalancer.project_id)
        cert_data = utils.get_cert_data(barbican_client, listener)
        try:
            if self.axapi_client.file.ssl_cert.exists(file=cert_data.cert_filename):
                self.axapi_client.file.ssl_cert.delete(file=cert_data.cert_filename)
                LOG.debug("SSL certificate deleted successfully: %s", cert_data.cert_filename)
        except Exception as e:
            LOG.warning("Failed to delete SSL Certificate: %s", str(e))
        return cert_data


class SSLKeyDelete(task.Task):
    """Task to delete an SSL Key"""
    @axapi_client_decorator
    def execute(self, cert_data, vthunder):
        try:
            if self.axapi_client.file.ssl_key.exists(file=cert_data.key_filename):
                self.axapi_client.file.ssl_key.delete(file=cert_data.key_filename)
                LOG.debug("SSL Key deleted successfully: %s", cert_data.key_filename)
        except Exception as e:
            LOG.warning("Failed to delete SSL Key: %s", str(e))


class ClientSSLTemplateDelete(task.Task):
    """Task to delete a client ssl template for a listener"""
    @axapi_client_decorator
    def execute(self, cert_data, vthunder):
        try:
            if self.axapi_client.slb.template.client_ssl.exists(name=cert_data.template_name):
                self.axapi_client.slb.template.client_ssl.delete(name=cert_data.template_name)
                LOG.debug("Client SSL Template deleted successfully: %s",
                          cert_data.template_name)
        except Exception as e:
            LOG.warning("Failed to delete Client SSL Template: %s", str(e))



# Copyright 2019 A10 Networks
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""A10 Octavia Helper Module

"""
import netaddr
from oslo_log import log as logging

from a10_octavia.common import data_models

LOG = logging.getLogger(__name__)


def validate_ipv4(address):
    """Validates for IP4 address format"""
    if not netaddr.valid_ipv4(address, netaddr.core.INET_PTON):
        return False
    return True


def validate_params(rack_info):
    """Check for all the required parameters for rack configurations.
    """
    if all(k in rack_info for k in ('project_id', 'ip_address',
                                    'username', 'password', 'device_name')):
        if all(rack_info[x] is not None for x in ('project_id', 'ip_address',
                                                  'username', 'password', 'device_name')):
            if validate_ipv4(rack_info['ip_address']):
                return True
            LOG.error('Invalid IP address given ' + rack_info['ip_address'])
    LOG.error('Configuration of `devices` under [RACK_VTHUNDER] is invalid. '
              'Please check your configuration. The params `project_id`, '
              '`ip_address`, `username`, `password` and `device_name` cannot be None ')
    return False


def convert_to_rack_vthunder_conf(rack_list):
    """ Validates for all vthunder nouns for rack devices
        configurations.
    """
    rack_dict = {}
    validation_flag = False
    try:
        for rack_device in rack_list:
            validation_flag = validate_params(rack_device)
            if validation_flag:
                rack_device['undercloud'] = True
                vthunder_conf = data_models.VThunder(**rack_device)
                rack_dict[rack_device['project_id']] = vthunder_conf
            else:
                LOG.warning('Invalid definition of rack device for '
                            'project ' + rack_device['project_id'])

    except KeyError as err:
        LOG.error("Invalid definition of rack device in configuration file."
                  "The Loadbalancer you create shall boot as overcloud."
                  "Check attribute: " + str(err))
    return rack_dict
